//
//		MP3 Micro
//	 	mini SD card MP3 Player
//
//
//	MCU:		ATmega168		FUSE bit: D1EFh (external clock)
//	Compiler:	AVR-GCC
//
//	2005.7.1	Dream Computer Inc.
//

#define BV(bit)			(1<<(bit))
#define cbi(addr,bit)	addr&=~_BV(bit)
#define sbi(addr,bit)	addr|=_BV(bit)
#define	F_CPU			4000000UL	// NbNg(Hz)

#include	<stdio.h>
#include	<stdlib.h>
#include	<avr/io.h>
#include	<avr/signal.h>
#include	<avr/interrupt.h>
#include	<avr/pgmspace.h>
#include	<avr/wdt.h>
#include	<avr/delay.h>
#include	"loud.c"

#define		NOP		asm volatile("nop\n"::)
#define		SLEEP	asm volatile("sleep\n"::)

// VS1011 SCI MODE BIT
#define		SM_DIFF			0	// Differential 0 normal in-phase audio 1 left channel inverted
#define		SM_SETTOZERO	1	// Set to zero 0 right 1 wrong
#define		SM_RESET		2	// Soft reset 0 no reset 1 reset
#define		SM_OUTOFWAV		3	// Jump out of WAV decoding 0 no 1 yes
#define		SM_PDOWN		4	// Powerdown 0 power on 1 powerdown
#define		SM_TESTS		5	// Allow SDI tests 0 not allowed 1 allowed
#define		SM_STREAM		6	// Stream mode 0 no 1 yes
#define		SM_SETTOZERO1	7	// set to zero 0 right 1 wrong, this isnft zero
#define		SM_DACT			8	// DCLK active edge 0 rising 1 falling
#define		SM_SDIORD		9	// SDI bit order 0 MSb first 1 MSb last
#define		SM_SDISHARE		10	// Share SPI chip select 0 no 1 yes
#define		SM_SDINEW		11	// VS1002 native SPI modes 0 no 1 yes
#define		SM_SETTOZERO2	12	// set to zero 0 right 1 wrong, this isnft zero
#define		SM_SETTOZERO3	13	// set to zero 0 right 1 wrong, this isnft zero

// VS1011 SCI Registers
#define		SCI_MODE		0x0	// Mode control.(rw)
#define		SCI_STATUS		0x1	// Status of VS1011.(rw)
#define		SCI_BASS		0x2	// Built-in bass enhancer.(rw)
#define		SCI_CLOCKF		0x3	// Clock freq + doubler.(rw)
#define		SCI_DECODE		0x4	// TIME Decode time in seconds.(r)
#define		SCI_AUDATA		0x5	// Misc. audio data.(rw)
#define		SCI_WRAM		0x6	// RAM write.(rw)
#define		SCI_WRAMADDR	0x7	// Base address for RAM write.(rw)
#define		SCI_HDAT0		0x8	// Stream header data 0.(r)
#define		SCI_HDAT1		0x9	// Stream header data 1.(r)
#define		SCI_AIADDR		0xA	// Start address of application.(rw)
#define		SCI_VOL			0xB	// Volume control.(rw)
#define		SCI_AICTRL0		0xC	// Application control register 0.(rw)
#define		SCI_AICTRL1		0xD	// Application control register 1.(rw)
#define		SCI_AICTRL2		0xE	// Application control register 2.(rw)
#define		SCI_AICTRL3		0xF	// Application control register 3.(rw)

#define		VS1011_CLOCK	12288000	// VS1011 Uq̎g(Hz)

#define		PB_LCD_CS		0	// PORTB
#define		PB_SD_CS		1	// PORTB
#define		PB_VS1011_XCS	2	// PORTB
#define		PB_SPI_DOUT		3	// PORTB
#define		PB_SPI_DIN		4	// PORTB
#define		PB_SPI_CLK		5	// PORTB
#define		PB_EL_CS		6	// PORTB
#define		PB_LED2			7	// PORTB

#define		PC_SW_RIGHT		0	// PORTC
#define		PC_SW_LEFT		1	// PORTC
#define		PC_VS1011_XDCS	2	// PORTC
#define		PC_VS1011_DREQ	3	// PORTC
#define		PC_VS1011_RST	4	// PORTC
#define		PC_SD_SW2		5	// PORTC
#define		PC_AVR_RST		6	// PORTC

#define		PD_SW_KEY4		1	// PORTD
#define		PD_SW_KEY3		2	// PORTD
#define		PD_SW_KEY2		3	// PORTD
#define		PD_SW_KEY1		4	// PORTD
#define		PD_LED1			5	// PORTD
#define		PD_LCD_RS		6	// PORTD
#define		PD_LCD_RST		7	// PORTD

// KeyStatus̃rbg
#define		SW_START_STOP	0
#define		SW_BOTH			1
//#define		SW_REW			2
//#define		SW_FF			3
//#define		SW_VOLDOWN		4
//#define		SW_VOLUP		5

// PlayModẽrbg
#define		MOD_RND			0
#define		MOD_REP_ONE		1
#define		MOD_REP_ALL		2
#define		MOD_SEARCH_FF	3
#define		MOD_SEARCH_REW	4
#define		MOD_FF			5
#define		MOD_REW			6


#define		BEEP_HIGH		0
#define		BEEP_LOW		1

// EEPROM address assignment
#define		EEPROM_SongNum		1	// Song number on play
#define		EEPROM_CurrCluster1	2	// Cluster(LOW) on play
#define		EEPROM_CurrCluster2	3	// Cluster(HIGH) on play
#define		EEPROM_RemainSect1	4	// Remain sector on play
#define		EEPROM_RemainSect2	5	// Remain sector on play
#define		EEPROM_RemainSect3	6	// Remain sector on play
#define		EEPROM_SongSize1	8	// Song size(byte) on play
#define		EEPROM_SongSize2	9	// Song size(byte) on play
#define		EEPROM_SongSize3	10	// Song size(byte) on play
#define		EEPROM_SongSize4	11	// Song size(byte) on play
#define		EEPROM_Volume		12	// Volume
#define		EEPROM_Speed		13	// Play speed
#define		EEPROM_Loud			14	// Loudness pattern No.
#define		EEPROM_PlayMode		15	// Play mode
#define		EEPROM_Timer		16	// Timer for random mode

/*
// Battery voltage
#define		BATT_15			500		// 1.5V
#define		BATT_14			467		// 1.4V
#define		BATT_13			433		// 1.3V
#define		BATT_12			400		// 1.2V
#define		BATT_11			367		// 1.1V
#define		BATT_10			333		// 1.0V
#define		BATT_09			300		// 0.9V
#define		BATT_08			267		// 0.8V
*/



PROGMEM uint8_t BeepHigh[] = { 0x53, 0xEF, 0x6E, 0x24, 0, 0, 0, 0 };	// 1500Hz
PROGMEM uint8_t Beeplow[] = { 0x53, 0xEF, 0x6E, 0x21, 0, 0, 0, 0 };		// 375Hz
PROGMEM uint8_t BeepStop[] = { 0x45, 0x78, 0x69, 0x74, 0, 0, 0, 0 };	// beep stop


struct SongInfo {
	uint8_t		FATtype;				// 0:FAT12  1:FAT16
	uint8_t		SectorsPerCluster;		// Sector/cluster
	uint16_t	RootDirEntriesCount;	// Root dir entries Count
	uint16_t	FATstart;				// Fat start sector
	uint16_t	DIRstart;				// Root dir start sector
	uint32_t	DataStart;				// Data area start sector
	uint16_t	Cluster;				// Current cluster
	uint16_t	SongNum;				// Song number on play
	uint16_t	TotalSong;				// Total song
	uint32_t	SongSize;				// Song size'(byte) on play
	uint32_t	RemainSect;				// Remaining sector on play
} Song;


volatile uint8_t	SearchLen;		// Search mode song size
volatile uint8_t	Volume;			// Sound volume(0:min att, 0xff:max att)
volatile uint8_t	Speed;			// Play speed(-128:min speed, 127:max speed)
volatile uint8_t	Loud;			// Loudness pattern No.
volatile int16_t	Vbatt;			// Battery voltage
volatile uint8_t	KeyStatus;		// b0:start/stop, b2:key both
volatile uint8_t	PlayMode;		// b0:normal/random, b1:repeat song, b2:repeat all
volatile uint8_t	KeyMode;		// mode 0: key_right -> Vol up			key_left -> Vol down
								// mode 1: key_right -> Forword step	key_left -> Backword step
								// mode 2: key_right -> FF				key_left -> REW
								// mode 3: key_right -> Intro Search on	key_left -> Intro search off
								// mode 4: key_right -> Random mode		key_left -> Normal mode
								// mode 5: key_right -> Repeat one		key_left -> Single playl
								// mode 6: key_right -> Repeat all		key_left -> Single playl
								// mode 7: key_right -> Play speed up	key_left -> Play speed down
								// mode 8: key_right -> Loudness on		key_left -> Loudness off
								// mode 9: key_right -> Battery check

volatile uint8_t	SwLeftCount;	// Right key counter
volatile uint8_t	SwRightCount;	// Left key counter
volatile uint8_t	SwBothCount;	// Both Right and Left key counter
volatile uint16_t	ModeTimeout;	// Key Timeout counter(2 sec)
volatile uint16_t	Timer;			// Shutdown Timeout counter(1 sec)
volatile uint8_t	SwReleaseCount;	// Key release timer

uint8_t		SectBuff[512];			// SD card sector read buffer
uint8_t		Buff[256];				// Temporary buffer
uint32_t	BuffPtr;				// obt@|C^


uint8_t		SD_Init(void);
uint8_t		SD_open(uint32_t adrs);
void		SD_close(void);
uint8_t		SD_command(uint8_t com, uint32_t arg);
uint8_t		SPI_in(void);
void		SPI_out(uint8_t data);

uint16_t	read_word(uint32_t addr);
void		read_32(uint32_t addr);
uint8_t		read_FAT16_info(struct SongInfo *Song);

uint16_t 	search_MP3(uint16_t num);
int8_t		play_song(struct SongInfo *Song);
uint16_t 	next_cluster(uint16_t c);
uint16_t	check_cont_cluster(uint16_t c);
void		beep(uint8_t n, int8_t err);

void		vs1011_sin(PGM_VOID_P data);
void		vs1011_init(void);
void		VS1011_command(uint8_t addr, uint16_t arg);
uint16_t	VS1011_status(uint8_t addr);

void		PowerDown(void);
//void		check_batt(void);
void		write_eeprom(uint8_t adrs, uint8_t data);
uint8_t		read_eeprom(uint8_t adrs);
int8_t		KeyRight(void);
int8_t		KeyLeft(void);
void 		init_eeprom(void);
void 		save_eeprom(struct SongInfo *Song);
void		restore_eeprom(struct SongInfo *Song);
void		restore_loud(void);
void		vs1011_wram(PGM_VOID_P Data, uint16_t size);


//*********************************************************
//	SD card initialize
//*********************************************************
uint8_t SD_Init(void){
	uint8_t	c;

	sbi(PORTB, PB_SD_CS);					// SD not select

	for (c=0; c < 20; c++){
		SPI_in();
	}

	cbi(PORTB, PB_SD_CS);		// SD select
	c = SD_command(0, 0);		// Select SPI mode

	while ((c != 1)&&(c != 0xFF)){
		SLEEP;
		c = SD_command(0, 0);	// Repeat SPI mode command
	}
	while (c == 1){
		c = SD_command(1, 0);	// issue command 1
	}
	sbi(PORTB, PB_SD_CS);		// SD not select

	return c;
}


/*********************************************************
Read FAT16 info

MBR (Master Boot Recode)
		0-445	checkRoutionOnx86[446];
        0		bootDescriptor;             // 0x80: bootable device, 0x00: non-bootable
        1-3		firstPartitionSector[3];    // 1st sector number
        4		fileSystemDescriptor;       // 1:FAT12, 4:FAT16(less than 32MB), 5:g DOS p[eBV,
                                               6:FAT16(more 32MB), 0xb:FAT32(more 2GB),
                                               0xc:FAT32 Int32h g, 0xe:FAT16 Int32h g,
                                               0xf:g DOS p[eBV Int32h g
        5-7		lastPartitionSector[3];
        8-11	firstSectorNumbers[4];      // first sector number (link to BPB sector)
        12-15	numberOfSectors[4];
	    16-17	sig[2];                     // 0x55, 0xaa

BPB (BIOS Parameter Block)
        0-2		jmpOpeCode[3];          // 0xeb ?? 0x90
        3-10	OEMName[8];
        11-12	bytesPerSector[2];      // bytes/sector
        13		sectorsPerCluster;      // sectors/cluster
        14-15	reservedSectors[2];     // reserved sector, beginning with sector 0
        16		numberOfFATs;           // file allocation table
        17-18	rootEntries[2];         // root entry (512)
        19-20	totalSectors[2];        // partion total secter
        21		mediaDescriptor;        // 0xf8: Hard Disk
        22-23	sectorsPerFAT[2];       // sector/FAT (FAT32 always zero: see bigSectorsPerFAT)
        24-25	sectorsPerTrack[2];     // sector/track (not use)
        26-27	heads[2];               // heads number (not use)
        28-31	hiddenSectors[4];       // hidden sector number
        32-35	bigTotalSectors[4];     // total sector number
        // info
        36		driveNumber;
        37		unused;
        38		extBootSignature;
        39-42	serialNumber[4];
        43-53	volumeLabel[11];
        54-61	fileSystemType[8];      // "FAT16   "
        62-509	loadProgramCode[448];
        510-511	sig[2];                 // 0x55, 0xaa

	input: 	none
	output:
		bpb
		SectorsPerCluster
		FATstart
		RootDirEntriesCount
		DIRstart
		DataStart
		FATtype
	return:	ok 0
			ng 1, 2
*********************************************************/
uint8_t read_FAT16_info(struct SongInfo *Song){
	uint32_t	sect;
	uint16_t	bpb;
	uint16_t	n;

	// Read FAT16 info. sector size=512bytes
	read_32(446);											// Read MBR(0x1be)
	bpb = (uint16_t)Buff[8] + ((uint16_t)Buff[9]<<8);		// BPB start sector
	read_32((uint32_t)bpb * 512);							// Read BPB(32byte)
	n = (uint16_t)Buff[11] + ((uint16_t)Buff[12] << 8);		// n:bytes per Sector

	if (n != 512){
		return 2;											// Sector size != 512byte
	}

	Song->SectorsPerCluster = Buff[13];						// sectors per Cluster
	Song->FATstart = bpb + (uint16_t)Buff[14]
		+ ((uint16_t)Buff[15] << 8);						// FAT start sector
	Song->RootDirEntriesCount = (uint16_t)Buff[17]
		+ ((uint16_t)Buff[18] << 8);						// root entry number
	n = (uint16_t)Buff[22] + ((uint16_t)Buff[23] << 8);			// sectors per FAT

	if (n == 0){										// FAT32?
		return 1;										// Yes
	}

	Song->DIRstart = (uint16_t)Song->FATstart + (uint16_t)Buff[16] * n;	// DirEntries start sector
	n = Song->RootDirEntriesCount / 16;						// sectors par RootDirEntries

	if (Song->RootDirEntriesCount % 16){
		n++;	// ]肪o؂グ
	}

	Song->DataStart = (uint32_t)Song->DIRstart + (uint32_t)n;		// Data area start sector

	sect = (uint32_t)Buff[19] + ((uint32_t)Buff[20] << 8);	// partition total sector
	if (sect == 0){										// FAT32
		read_32((uint32_t)bpb * 512 + 32);
		sect = (uint32_t)Buff[0] + ((uint32_t)Buff[1] << 8)
			+ ((uint32_t)Buff[2] << 16) + ((uint32_t)Buff[3] << 24);
	}

	sect -= Song->FATstart;
	sect /= Song->SectorsPerCluster;

	if (sect >= 0x1000)
		Song->FATtype = 1;	// FAT16
	else
		Song->FATtype = 0;	// FAT12

	return 0;
}


/*********************************************************
	Search MP3 files
	inputF
		num:	search songs
		̐ȉłĂfBNgGgSĒ׏I߂
	returnF
		searched number
		Buff[0`31]	last song directory info

Directory Entries
		0-7		name[8];            // file name
									name[0]
									0x00  empty
									0x05  deleted
									0x2e  directory entry
									      0x2e (.)
									      0x2e, 0x2e (..)
									0xe5  deleted
		8-10	extension[3];       // file name extension
		11		attribute;          // file attribute
									bit 4    directory flag
									bit 3    volume flag
									bit 2    hidden flag
									bit 1    system flag
									bit 0    read only flag
									0x20	archive entry
									0x10	directory entry
									0x08	volume label entry
									0x0f	int32_t file name entry
		12		reserved;           // use NT or same OS
		13		createTimeMs;       // FAT16 Ŏgpt@C쐬10~b (0 ` 199)
		14-15	createTime[2];      // FAT16 Ŏgpt@C쐬
		16-17	createDate[2];      // FAT16 Ŏgpt@C쐬t
		18-19	accessDate[2];      // FAT16 Ŏgpt@CEANZXt
		20-21	clusterHighWord[2]; // FAT32 ŎgpNX^ԍ̏ 16 bits
		22-23	updateTime[2];
		24-25	updateDate[2];
		26-27	cluster[2];         // start cluster number
		28-31	fileSize[4];        // file size in bytes (directory is always zero)

*********************************************************/
uint16_t search_MP3(uint16_t num){
	uint16_t	ent;
	uint32_t	sect;
	uint8_t		i, j;
	uint16_t	n;

	n = 0;
	sect = (uint32_t)Song.DIRstart * 512;
	for (ent = 0; ent < Song.RootDirEntriesCount; ){
		if (SD_open(sect))					// SDJ[h̓ǏoI[v
			return n;						// sA܂łɌoȐԂ

		cli();

		for (i = 0; i < 16; i++){			// PZN^̃f[^(512oCg)Ǎŏ
			for (j = 0; j < 32; j++){		// PGg[i32oCgj̃f[^ǂݍ
				Buff[j] = SPI_in();
			}
			if (Buff[0] == 0)				// gp̈oŐ؂
				break;
			if ( ((Buff[26] != 0)||(Buff[27] != 0))&&					// NX^ԍ0ł͂Ȃ
				(Buff[0] != 0xE5)&&										// f[g}[N͕tĂȂ
				(!(Buff[11] & 0x18))&&									// Song file
				((Buff[8]=='M')&&(Buff[9]=='P')&&(Buff[10]=='3')) ) {	// gq"MP3"
																
				n++;
				if (--num == 0)				// wȐoŐ؂
					break;
			}
			ent++;
		}

		for (++i; i < 16; i++){				// ZN^̎cǂݎ̂
			for (j = 0; j < 32; j++){
				SPI_in();
			}
		}

		sei();

		SD_close();
		if (Buff[0] == 0)	// gp̈oŐ؂
			break;
		if (num == 0)		// wȐoŐ؂
			break;
		sect += 512;		// ̃ZN^
	}

	return n;				// oȐԂ
}


/****************************************
	Play MP3 songs
	SongNumber:		song number
	StartCluster:	start cluster, if 0 play from beginningt
	Remain_sect: 	Number of remainder sectors

	input:	SongNumber....song number
			StartCluster..start cluster
			Remain_sect
	output:	none
	return:	 0....ok
			 1....Stop switch was pushed.
			 2....Ff switch was pushed.
			 3....Rew switch was pushed.
			-1... no song
			-2....error
			
****************************************/
int8_t play_song(struct SongInfo *Song){
	uint32_t	sect;
	uint16_t	RemainByte;
	uint16_t	cn;
	uint16_t	last_cluster;
	uint16_t	*clust_buff;
	uint16_t	c;
	uint8_t		*p;
	uint16_t	i, j, k;
	uint8_t		Vol_offset, offset;


	if ( Song->SongNum != search_MP3(Song->SongNum) ){					// SongNumԍ̋ȂT
		return -1;														// Ȃ-1ԂďI
	}

	Song->SongSize = (uint32_t)Buff[28] + ((uint32_t)Buff[29]<<8)		// Ȃ̃oCg
			  + ((uint32_t)Buff[30]<<16) + ((uint32_t)Buff[31]<<24);

	if (Song->Cluster){													// Ȃ̓r̍Đ̏ꍇ
		RemainByte = Song->SongSize - (Song->RemainSect * 512);			// Ȃ̎coCg
	} else {															// Ȃ̍ŏ̍Đ̏ꍇ
		Song->Cluster = (uint16_t)Buff[26] + ((uint16_t)Buff[27] << 8);	// ȊJñNX^ԍ
		Song->RemainSect = Song->SongSize / 512;						// c̃ZN^͋Ȃ̃oCg/512
		RemainByte = Song->SongSize % 512;								// c̃oCg
	}

	if ((Song->Cluster == 0xFFFF)||(Song->Cluster < 2)){
		return -3;			// Cluster error
	}

	// Each consecutive cluster is stored in the buffer
	clust_buff = (uint16_t *)Buff;	// Assign Buff to cluster buffer
	c = Song->Cluster;
	BuffPtr = 0;			//obt@|C^Zbg
	for (i = 0; i < sizeof(Buff)/4; i++){	
		*clust_buff++ = c;
		*clust_buff++ = c = check_cont_cluster(c);
		c = next_cluster(c);
		if (c == 0xFFFF){
			*clust_buff++ = c;
			*clust_buff = c;
			break;
		}
	}


	// First cluster block
	clust_buff = (uint16_t *)Buff;
	Song->Cluster = *clust_buff++;
	last_cluster = *clust_buff++;

	cn = Song->SectorsPerCluster;
	sect = (((uint32_t)Song->Cluster - 2) * (uint32_t)Song->SectorsPerCluster + Song->DataStart) * 512;

	if ((PlayMode & (1<<MOD_SEARCH_FF)) || (PlayMode & (1<<MOD_SEARCH_REW))) {
		Song->RemainSect = 40*SearchLen;
		RemainByte = 0;
	}


	vs1011_init();		// ꂪȂIntro search̍ۃmCY

	while (Song->RemainSect > 0){

/*	fobOpBL[[hA^Cɕ\
	if (KeyMode == 0) {
		PORTD &= ~(1<<PD_LED1);	// PD_LED1o
		PORTB |= (1<<PB_LED2);	// PB_LED2o
	} else {
		PORTB &= ~(1<<PB_LED2);	// PB_LED2o
		PORTD |= (1<<PD_LED1);	// PD_LED1o
	}
*/

		if (Volume > 0x80) {			// {[_Eł̃p[It ?
			VS1011_command(SCI_VOL, (0x30 << 8)| 0x30);	// BEEP̂߂̃{[Zbg
				return 5;				// Yes
		}

		if (KeyStatus & (1<<SW_START_STOP)) {
			KeyStatus &= ~(1<<SW_START_STOP);
			return 1;					// WOvbVł̃p[It
		}

		if (Loud == 0)					// EhlXs̉ʕ␳
			offset = 0;
		else if (Loud == 1)
			offset = 10 * 2;
		else if (Loud == 2)
			offset = 5 * 2;
		else if (Loud == 3)
			offset = 5 * 2;
		else if (Loud == 4)
			offset = 3 * 2;
		else if (Loud == 5)
			offset = 5 * 2;
		else if (Loud == 6)
			offset = 3 * 2;
		else if (Loud == 7)
			offset = 5 * 2;
		else if (Loud == 8)
			offset = 0;
		else if (Loud == 9)
			offset = 5 * 2;
		else if (Loud == 10)
			offset = 3 * 2;
		else if (Loud == 11)
			offset = 5 * 2;
		else if (Loud == 12)
			offset = 0;

		Vol_offset = (Volume > offset) ? Volume - offset : 0;
		VS1011_command(SCI_VOL, ((uint16_t)Vol_offset << 8)|((uint16_t)Vol_offset));	// Issue volume control(Right & Left)

		if (SwBothCount) {								// WOvbVIntro search̋Ȍ
			if ((PlayMode & (1<<MOD_SEARCH_FF)) || (PlayMode & (1<<MOD_SEARCH_REW))) {
				while ( KeyRight() || KeyLeft()  )		// WOvbṼ[X҂
					;
				return 4;
			}
		}

		else {								// WOvbVȂꍇ
			if ( KeyMode == 1 ){			// Forword step or Backword step
				if (SwRightCount > 2) {
					beep(1, BEEP_HIGH);
					SwRightCount = 0;
					return 2;					// Ȃi߂
				} else if (SwLeftCount > 2) {
					beep(1, BEEP_HIGH);
					SwLeftCount = 0;
					return 3;					// Ȃ߂
				}
			}

			else if ( KeyMode == 2 ) {			// FF or REW
				if (SwRightCount) {					// FF?
					PlayMode |= (1<<MOD_FF);
				} else if (SwLeftCount) {			// REW?
					PlayMode |= (1<<MOD_REW);
					beep(1, BEEP_HIGH);
					SwLeftCount = 0;
					return 3;					// Ȃ߂
				} else {
					PlayMode &= ~(1<<MOD_FF);
					PlayMode &= ~(1<<MOD_REW);
				}
			}

			else if ( KeyMode == 3 ) {			// Intro search
				if (SwRightCount > 2) {					// Search FF?
					beep(1, BEEP_HIGH);					// Yes
					PlayMode &= ~(1<<MOD_SEARCH_REW);
					PlayMode |= (1<<MOD_SEARCH_FF);
					SearchLen++;
					return 2;							// FF
				} else if (SwLeftCount > 2) {			// Search REW?
					beep(1, BEEP_HIGH);					// Yes
					PlayMode &= ~(1<<MOD_SEARCH_FF);
					PlayMode |= (1<<MOD_SEARCH_REW);
					SearchLen++;
					return 3;							// REW
				}
			}

			else if ( KeyMode == 4 ) {			// Random ?
				if (SwRightCount > 4) {					// RND?
					beep(1, BEEP_HIGH);					// Yes
					PlayMode |= (1<<MOD_RND);
				} else if (SwLeftCount > 4) {			// Normal?
					beep(1, BEEP_HIGH);					// Yes
					PlayMode &= ~(1<<MOD_RND);
				}
			}

			else if ( KeyMode == 5 ) {			// Repeat one ?
				if (SwRightCount > 4) {					// Repeat song mode?
					beep(1, BEEP_HIGH);					// Yes
					PlayMode |= (1<<MOD_REP_ONE);
					PlayMode &= ~(1<<MOD_REP_ALL);
				} else if (SwLeftCount > 4) {			// Single play?
					beep(1, BEEP_HIGH);					// Yes
					PlayMode &= ~(1<<MOD_REP_ONE);
					PlayMode &= ~(1<<MOD_REP_ALL);
				}
			}

			else if ( KeyMode == 6 ) {			// Repeat all ?
				if (SwRightCount > 4) {					// Repeat All mode?
					beep(1, BEEP_HIGH);					// Yes
					PlayMode |= (1<<MOD_REP_ALL);
					PlayMode &= ~(1<<MOD_REP_ONE);
				} else if (SwLeftCount > 4) {			// Single play?
					beep(1, BEEP_HIGH);					// Yes
					PlayMode &= ~(1<<MOD_REP_ONE);
					PlayMode &= ~(1<<MOD_REP_ALL);
				}
			}

			else if ( KeyMode == 7 ) {			// Play speed control
				if (SwRightCount > 2 || SwLeftCount > 2) {	// Speed up or down ?
					if (SwRightCount > 2) {					// Speed down ?
						if (Speed++ == 10)					// Yes
							Speed = 10;
					} else if (SwLeftCount > 2) {			// Speed up ?
						if (Speed-- == 0)					// Yes
							Speed = 0;
					}
					beep(1, BEEP_HIGH);
				}
			}

			else if ( KeyMode == 8 ) {			// Loudness
				if (SwRightCount > 2 || SwLeftCount > 2) {	// Loudness on or off ?
					if (SwRightCount > 2) {					// Loudness no. up ?
						if (Loud++ == 12)					// Yes
							Loud = 12;
					} else if (SwLeftCount > 2) {			// Loudness no. down ?
						if (Loud-- == 0)					// Yes
							Loud = 0;
					}
					beep(1, BEEP_HIGH);
				}
			}

			else if ( KeyMode == 9 ) {			// Battery voltage check
/********************************************
				if (SwRightCount > 2 || SwLeftCount > 2) {	// Check battery voltage
					j = BEEP_HIGH;
					if (Vbatt > BATT_15) {			// Battery voltage: >1.5V
						i = 5;
					} else if (Vbatt > BATT_14) {	// Battery voltage: 1.4V
						i = 4;
					} else if (Vbatt > BATT_13) {	// Battery voltage: 1.3V
						i = 3;
					} else if (Vbatt > BATT_12) {	// Battery voltage: 1.2V
						i = 2;
					} else if (Vbatt > BATT_11) {	// Battery voltage: 1.1V
						i = 1;
					} else if (Vbatt > BATT_10) {	// Battery voltage: 1.0V
						i = 3;
						j = BEEP_LOW;
					} else if (Vbatt > BATT_09) {	// Battery voltage: 0.9V
						i = 2;
						j = BEEP_LOW;
					} else {						// Battery voltage: <0.9V
						i = 1;
						j = BEEP_LOW;
					}

					beep(i, j);
					for (k = 0; k < 10*5; k++)		// Space 500ms
						SLEEP;
					beep(i, j);

				}
**********************************************/
			}
		}

		if (PlayMode & (1<<MOD_FF))
			;
		else {
			if (SD_open(sect))
				return -2;

			cli();
			for (p = SectBuff; p < (SectBuff + 512); )
				*p++ = SPI_in();

			SD_close();

			cbi(PORTC, PC_VS1011_XDCS);
			p = SectBuff;

			for (j = 0; j < 16; j++){
				while (!(PINC&(1<<PC_VS1011_DREQ))) {		// Check if VS1011 busy
					;
				}
				for (k = 0; k < 32; k++)
					SPI_out(*p++);
			}

			sbi(PORTC, PC_VS1011_XDCS);
			sei();
		}

		--Song->RemainSect;

		if (--cn > 0){
			sect += 512;	// ̃ZN^

		} else {			// ̃NX^

			if (++Song->Cluster > last_cluster){		// NX^io[𑝉BsAȏꍇFATēǍ
//	PORTB &= ~(1<<PB_LED2);	// PB_LED2o
				if ((void *)clust_buff < (void *)Buff + sizeof(Buff)){	// 
					Song->Cluster = *clust_buff++;	// NX^ԍobt@Ǎ
					last_cluster = *clust_buff++;

				} else {						// NX^ԍSDJ[hēǍ݂
					BuffPtr = 0;				// obt@|C^Zbg
					Song->Cluster = next_cluster(Song->Cluster);
					last_cluster = 0;
				}

				if ((Song->Cluster < 2)||(Song->Cluster == 0xFFFF)){	// ȃNX^ԍȂ珈Ő؂
					RemainByte = 0;
					break;
				}
			}

			sect = (((uint32_t)Song->Cluster - 2) * (uint32_t)Song->SectorsPerCluster + Song->DataStart) * 512;
			cn = Song->SectorsPerCluster;
		}
//		check_batt();	// obe[`FbN
	}


	if ((RemainByte > 0) && (RemainByte < 512)){	// cZN^TCY512oCgȉ̍Đ


		if (SD_open(sect))							// SDJ[hI[v
			return -2;								// SDJ[hǏoI[vs

		cli();

		for (p = SectBuff; p < (SectBuff + 512); )	// SDJ[h512oCgǏo
			*p++ = SPI_in();

		SD_close();									// SDJ[hN[Y

		cbi(PORTC, PC_VS1011_XDCS);					// VS1011XDCSLɂ
		p = SectBuff;

		for (j = 0; j < 16; j++){					// VS1011Ɏc̃f[^𑗂
			while (!(PINC&(1<<PC_VS1011_DREQ))) {	// VS1011DREQLɂȂ܂ő҂
				;
			}
			for (k = 0; k < 32; k++) {				// 32oCg1ubNƂăf[^𑗂
				if (RemainByte-- <= 0)
					break;
				SPI_out(*p++);
			}
		}

		sbi(PORTC, PC_VS1011_XDCS);					// VS1011XDCS𖳌ɂ
		sei();
	}

	if (PlayMode & (1<<MOD_SEARCH_FF))				// Intro search[hł̋Ȃ̏IBȂi߂
		return 2;							// FF
	else if (PlayMode & (1<<MOD_SEARCH_REW))		// Intro search[hł̋Ȃ̏IBȂ߂
		return 3;							// REW
	else 
		return 0;									// ʏI
}


//****************************************
//	Get next cluster
//
//	input:		NX^ԍ
//	output:		SectBuff[0, 511]
//	return:		̃NX^ԍ
//****************************************
uint16_t next_cluster(uint16_t c){
	uint32_t	addr;
	uint16_t	data;

	if (Song.FATtype == 0){	// FAT12
		addr = (uint32_t)Song.FATstart * 512 + (uint32_t)c + (c >> 1);
		if ((addr & 0x1FF) == 0x1FF){
			data = read_word(addr) & 0x00FF;
			data |= ((read_word(addr+1) & 0x00FF) << 8);
		} else {
			data = read_word(addr);
		}
		if (c & 1){
			c = data >> 4;
		} else {
			c = data & 0x0fff;
		}
		if (c >= 0x0ff8)
			c = 0xFFFF;
	}

	else {		// FAT16
		addr = (uint32_t)Song.FATstart * 512 + (uint32_t)c * 2;
		c = read_word(addr);
		if (c >= 0xFFF8)			// end cluster
			c = 0xFFFF;
	}

	return c;
}


//****************************************
//	Check if cluster continued
//
//	input:		NX^ԍ
//	output:		SectBuff[0, 511]
//	return:		NX^ԍȀꍇ͎̃NX^ԍic+1)
//				NX^ԍsȀꍇ͂̂܂܂̃NX^ԍ(c)
//
//****************************************
uint16_t check_cont_cluster(uint16_t c){
	uint16_t	next;

	for(;;) {
		next = next_cluster(c);
		if (++c != next){
			--c;
			break;
		}
	}

	return c;
}


//****************************************
//	Read 2byte from SD card
//
//	input:		address
//	output:		SectBuff[0, 511]
//	return:		ok   word data
//				ng   0
//****************************************
uint16_t read_word(uint32_t addr){
	uint16_t	w;
	uint16_t	b;
	uint8_t	*p;

	b = addr & 0x1FF;
	addr &= 0xFFFFFE00;


	if (addr != BuffPtr){
		if (SD_open(addr))
			return 0;

		cli();
		for (p = SectBuff; p < (SectBuff + 512); ){
			*p++ = SPI_in();
		}
		sei();
		SD_close();
		BuffPtr = addr;
	}

	w = (uint16_t)SectBuff[b++];
	w |= ((uint16_t)SectBuff[b] << 8);

	return w;
}


//****************************************
//	Read 32byte from SD card
//
//	input:		address
//	output:		Buff[0,31]
//	return:		none
//****************************************
void read_32(uint32_t addr){
	uint16_t		i, b;

	if (SD_open(addr & 0xFFFFFE00))
		return;
	b = (uint16_t)addr & 0x01FF;

	cli();

	for (i=0; i < 512; i++){
		if (i == b){
			int16_t	n;
			for (n = 0; n < 32; n++){
				Buff[n] = SPI_in();
				i++;
			}
		} else {
			SPI_in();
		}
	}
	sei();
	SD_close();
}


//*******************************************************
//	Check battery voltage
//*******************************************************
/*
void check_batt(void){
	if (Vbatt < BATT_08){
		beep(1, BEEP_LOW);
	}
}
*/

//*********************************************************
//	Shutdown handler
//*********************************************************
void PowerDown(void){

	PCIFR = 0;

	SMCR = (1<<SM1)|(1<<SE);			// Sleep mode is "power down"
	PCICR = 1<<PCIE1;
	PCMSK1 = (1<<PCINT8)|(1<<PCINT9);	// Detect change of Key_Right, key_left

	sei();
	SLEEP;
}


//*******************************************************
//	Restore Loudness code
//*******************************************************
void vs1011_wram(PGM_VOID_P Data, uint16_t size){
	uint16_t	i;

	VS1011_command(SCI_WRAMADDR, pgm_read_word(Data++));	// Issue RAM address
	Data++;

	for (i = 0; i < size-1; i++){
		VS1011_command(SCI_WRAM, pgm_read_word(Data++));	// Issue RAM data
		Data++;
	}
}


void restore_loud(void){
	vs1011_wram(loud1, sizeof(loud1)/2);
	vs1011_wram(loud2, sizeof(loud2)/2);
	vs1011_wram(loud3, sizeof(loud3)/2);
	vs1011_wram(loud4, sizeof(loud4)/2);

	VS1011_command(SCI_AIADDR, 0x200);		// Activate loudness code
	VS1011_command(SCI_AICTRL0, Loud);		// Loudness off
}



//*******************************************************
//	Init EEPROM
//*******************************************************
void init_eeprom(void){
//	write_eeprom(EEPROM_Volume, 0x30);
	write_eeprom(EEPROM_Speed, 0);
//	write_eeprom(EEPROM_Loud, 0);
	write_eeprom(EEPROM_SongNum, 1);
	write_eeprom(EEPROM_CurrCluster1, 0);
	write_eeprom(EEPROM_CurrCluster2, 0);
	write_eeprom(EEPROM_RemainSect1, 0);
	write_eeprom(EEPROM_RemainSect2, 0);
	write_eeprom(EEPROM_RemainSect3, 0);
	write_eeprom(EEPROM_SongSize1, 0);
	write_eeprom(EEPROM_SongSize2, 0);
	write_eeprom(EEPROM_SongSize3, 0);
	write_eeprom(EEPROM_SongSize4, 0);
	write_eeprom(EEPROM_PlayMode, 0);
}


//*******************************************************
//	Save EEPROM
//*******************************************************
void save_eeprom(struct SongInfo *Song){
	write_eeprom(EEPROM_Volume, Volume);
	write_eeprom(EEPROM_Speed, Speed);
	write_eeprom(EEPROM_Loud, Loud);
	write_eeprom(EEPROM_SongNum, Song->SongNum);
	write_eeprom(EEPROM_CurrCluster1, Song->Cluster);
	write_eeprom(EEPROM_CurrCluster2, Song->Cluster>>8);
	write_eeprom(EEPROM_RemainSect1, Song->RemainSect);
	write_eeprom(EEPROM_RemainSect2, Song->RemainSect>>8);
	write_eeprom(EEPROM_RemainSect3, Song->RemainSect>>16);
	write_eeprom(EEPROM_SongSize1, Song->SongSize);
	write_eeprom(EEPROM_SongSize2, Song->SongSize>>8);
	write_eeprom(EEPROM_SongSize3, Song->SongSize>>16);
	write_eeprom(EEPROM_SongSize4, Song->SongSize>>24);
	write_eeprom(EEPROM_PlayMode, PlayMode & 1<<MOD_RND);
	write_eeprom(EEPROM_Timer, Timer);
}


//*******************************************************
//	Write EEPROM
//*******************************************************
void write_eeprom(uint8_t adrs, uint8_t data){
	if (read_eeprom(adrs) != data){
		while(EECR & (1<<EEWE));
		EEARH = adrs >> 8;
		EEARL = adrs;
		EEDR = data;
		sbi(EECR, EEMWE);
		sbi(EECR, EEWE);
	}
}


//*******************************************************
//	Restore EEPROM
//*******************************************************
void restore_eeprom(struct SongInfo *Song){
	Volume = read_eeprom(EEPROM_Volume);
	Speed = read_eeprom(EEPROM_Speed);
	Loud = read_eeprom(EEPROM_Loud);
	Song->SongNum = read_eeprom(EEPROM_SongNum);
	Song->Cluster = (uint16_t)read_eeprom(EEPROM_CurrCluster2)<<8;
	Song->Cluster |= (uint16_t)read_eeprom(EEPROM_CurrCluster1);
	Song->RemainSect = (int32_t)read_eeprom(EEPROM_RemainSect3)<<16;
	Song->RemainSect |= (int32_t)read_eeprom(EEPROM_RemainSect2)<<8;
	Song->RemainSect |= (int32_t)read_eeprom(EEPROM_RemainSect1);
	Song->SongSize = (uint32_t)read_eeprom(EEPROM_SongSize4)<<24;
	Song->SongSize |= (uint32_t)read_eeprom(EEPROM_SongSize3)<<16;
	Song->SongSize |= (uint32_t)read_eeprom(EEPROM_SongSize2)<<8;
	Song->SongSize |= (uint32_t)read_eeprom(EEPROM_SongSize1);
	PlayMode = read_eeprom(EEPROM_PlayMode);
	Timer = read_eeprom(EEPROM_Timer);
}


//*******************************************************
//	Read EEPROM
//*******************************************************
uint8_t read_eeprom(uint8_t adrs){
	while(EECR & (1<<EEWE))
		;
	EEARH = adrs >> 8;
	EEARL = adrs;
	sbi(EECR, EERE);
	return EEDR;
}


//****************************************
//	BEEP
//	inputF	n	beep count
//	output: none
//****************************************
void beep(uint8_t n, int8_t flag){

	cli();

	VS1011_command(SCI_MODE, (1<< SM_RESET));			// Soft Reset
	while( !(PINC & 1<<PC_VS1011_DREQ) ) {				// Wait till DREQ high
		;
	}

	VS1011_command(SCI_CLOCKF, 0x8000 + (uint16_t)((VS1011_CLOCK)/2000));	// r[vp̍ĐxύX̃NbNg
	VS1011_command(SCI_MODE, (1<< SM_SDINEW) | (1<< SM_TESTS));				// TCĝ߃eXg[hɐݒ
	VS1011_command(SCI_VOL, (0x30 << 8)| 0x30);								// r[vp̕W

	for (; n > 0; --n){
		if (flag){
			vs1011_sin(Beeplow);
		} else {
			vs1011_sin(BeepHigh);
		}

		_delay_ms(40); 

		vs1011_sin(BeepStop);
		vs1011_sin(BeepStop);
	
		_delay_ms(40); 
	}

	sei();
	VS1011_command(SCI_CLOCKF, 0x8000 + (uint16_t)((VS1011_CLOCK)/20000*(Speed+10)));	// ĐxύX̃NbNg
	if ( Loud != 0 )
		VS1011_command(SCI_AIADDR, 0x200);	// EhlXR[hs
	else
		VS1011_command(SCI_AIADDR, 0);		// EhlXR[hpX
	VS1011_command(SCI_AICTRL0, Loud);		// EhlXԍZbgiO`PQj

}


//****************************************
// VS1011 initialize
//****************************************
void vs1011_init(void){

	VS1011_command(SCI_MODE, (1<< SM_RESET));			// Soft Reset
	while( !(PINC & 1<<PC_VS1011_DREQ) ) {				// Wait till DREQ high
		;
	}
	VS1011_command(SCI_MODE, (1<< SM_SDINEW));			// Restore reset, VS1002 native SPI mode 
	VS1011_command(SCI_CLOCKF, 0x8000 + (uint16_t)((VS1011_CLOCK)/20000*(Speed+10)));	// Clock set

	VS1011_command(SCI_AIADDR, 0x200);		// Activate loudness code
	VS1011_command(SCI_AICTRL0, 0);			// Loudness reset
	if (Loud == 0)							// Loudness off ?
		VS1011_command(SCI_AIADDR, 00);		// Yes
	else
		VS1011_command(SCI_AICTRL0, Loud);	// Set loudness No.
}


//****************************************
// VS1011 sine wave generation
//****************************************
void vs1011_sin(PGM_VOID_P data)
{

	uint8_t	i;
	uint8_t	c;

	for (i=0; i < 8; i++){
		c = (uint8_t)pgm_read_byte(data++);
		cbi(PORTC, PC_VS1011_XDCS);
		SPI_out(c);
		sbi(PORTC, PC_VS1011_XDCS);
	}
}


//****************************************
// VS1011 command write
//****************************************
void VS1011_command(uint8_t addr, uint16_t arg){

	cbi(PORTB, PB_VS1011_XCS);
	SPI_out(2);
	SPI_out(addr);
	SPI_out(arg >> 8);
	SPI_out(arg);
	sbi(PORTB, PB_VS1011_XCS);
}


//****************************************
// VS1011 status read
//****************************************
/*
uint16_t VS1011_status(uint8_t addr){
	uint16_t	r, r1;

	cbi(PORTB, PB_VS1011_XCS);
	SPI_out(3);
	SPI_out(addr);
	r = SPI_in();
	r1 = SPI_in();
	r = (r<<8) + r1;
	sbi(PORTB, PB_VS1011_XCS);
	return r;
}
*/



//****************************************
//	SDJ[hǏoR}h̔s
//
//	input:		ZN^ԍ
//	output:		
//	return:		ǏoR}h	0
//				ǏoR}hs	1
//****************************************
uint8_t SD_open(uint32_t adrs){
	uint16_t	i;
	uint8_t	r;
	uint8_t	retry;

	for (retry = 0; retry < 100; retry++){
		cbi(PORTB, PB_SD_CS);		// SD`bvZNgLɂ
		r = SD_command(17, adrs);	// SDPubN(512byte)̓ǏoR}hs
		if (r == 0){
			for (i = 0; i < 4000; i++){
				r = SPI_in();
				if (r == 0xFE){
					return 0;		// ǏoR}h
				}
			}
		} else if (r == 0xFE){
			return 0;				// ǏoR}h
		}
		sbi(PORTB, PB_SD_CS);		// SD`bvZNg𖳌ɂ
		SLEEP;
		SLEEP;
	}
	return 1;						// ǏoR}hs
}


//****************************************
//	SD card read close
//****************************************
void SD_close(void){
	uint16_t	i;

	SPI_in();	// CRC
	SPI_in();	// CRC

	for (i = 0; i < 1000; i++){
		if (SPI_in() == 0xFF)
			break;
	}
	sbi(PORTB, PB_SD_CS);		// SD not select
}


//****************************************
//	SD card command write
//
//	input:		command No., arg
//	output:		none
//	return:		ng  1
//				ok  0xFE
//****************************************
uint8_t SD_command(uint8_t com, uint32_t arg){
	uint16_t	i;
	uint8_t	r;

	for (i = 0; i < 1000; i++){
		r = SPI_in();
		if (r == 0xFF)
			break;
	}
	if (r != 0xFF)
		return 1;

	SPI_out(com|0x40);
	SPI_out(arg >> 24);
	SPI_out(arg >> 16);
	SPI_out(arg >> 8);
	SPI_out(arg);
	SPI_out(0x95);	// CRC

	for(i = 0; i < 1000; i++){
		r = SPI_in();
		if (r == 0xFE)
				break;
		if (!(r & 0x80)){
			SPI_in();	// need W clock
			break;
		}
	}

	return r;
}


//****************************************
//	SPI output
//****************************************
void SPI_out(uint8_t data){

	SPDR = data;
	while(!(SPSR & (1<<SPIF)))
		;
}


//****************************************
//	SPI input
//****************************************
uint8_t SPI_in(void){

	SPDR = 0xFF;
	while(!(SPSR & (1<<SPIF)))
		;
	return SPDR;
}


//****************************************
//	Check key on or off
//
//	input : none
//	return: 0........key off
//			1........key on
//****************************************
int8_t KeyRight(void){
	return ( !(PINC & 1<<PC_SW_LEFT) );
}


int8_t KeyLeft(void){
	return ( !(PINC & 1<<PC_SW_RIGHT) );
}


//*********************************
//	Main routine
//*********************************
int16_t main (void) {

	int8_t	r, c;


	MCUSR &= ~(1<<WDRF);	// EHb`hbO Zbg tO(WDRF)
	wdt_disable();			// EHb`hbO֎~

	PORTB = (1<<PB_SPI_DOUT)|(1<<PB_SPI_DIN)|(1<<PB_SPI_CLK);	// SPI|[gvAbv(mCYSDJ[h듮j
	PORTC = (1<<PC_SW_LEFT)|(1<<PC_SW_RIGHT);					// L[̓|[gvAbv
																// ȓd͂̂߁AeI/ONbN~
	PRR = (1<<PRTWI)|(1<<PRTIM2)|(1<<PRTIM0)|(1<<PRTIM1)|(1<<PRSPI)|(1<<PRUSART0)|(1<<PRADC);
	ACSR = (1<<ACD);			// ȓd͂̂߁AAiORp[^֎~
	PowerDown();				// L[͂܂ōŏd͂őҋ@



	// ̓X[v



	PRR &= ~((1<<PRTIM2)|(1<<PRSPI)|(1<<PRADC));	// ^C}/JE^2NbNSPINbN,ADCNbN𕜊
	CLKPR = (1<<CLKPCE);
//	CLKPR = 0;							// CPU CLOCK=8MHz
//	CLKPR = (1<<CLKPS0);				// CPU CLOCK=4MHz
	CLKPR = (1<<CLKPS1);				// CPU CLOCK=2MHz
//	CLKPR = (1<<CLKPS1)|(1<<CLKPS0);	// CPU CLOCK=1MHz

//	Port B initialize
	PORTB = ((1<<PB_LCD_CS)|(1<<PB_SD_CS)|(1<<PB_VS1011_XCS)|(1<<PB_SPI_DOUT)|
			(1<<PB_SPI_DIN)|(1<<PB_SPI_CLK)|(1<<PB_LED2));
	DDRB = (1<<PB_LCD_CS)|(1<<PB_SD_CS)|(1<<PB_VS1011_XCS)|(1<<PB_SPI_DOUT)|
			(1<<PB_SPI_CLK)|(1<<PB_EL_CS)|(1<<PB_LED2);

//	Port C initialize
	PORTC = (1<<PC_SW_LEFT)|(1<<PC_SW_RIGHT)|(1<<PC_VS1011_XDCS)|
			(1<<PC_VS1011_DREQ)|(1<<PC_VS1011_RST)|(1<<PC_SD_SW2);
	DDRC = (1<<PC_VS1011_XDCS)|(1<<PC_VS1011_RST);

//	Port D initialize
	PORTD = (1<<PD_SW_KEY4)|(1<<PD_SW_KEY3)|(1<<PD_SW_KEY2)|(1<<PD_SW_KEY1)|
			(1<< PD_LED1)|(1<<PD_LCD_RS)|(1<<PD_LCD_RST);
	DDRD = (1<<PD_LED1)|(1<<PD_LCD_RS)|(1<<PD_LCD_RST);



//	SPCR = 0x52;	// SPI}X^, CPOL=0, CPHA=0, Clock 8MHz/64=125kHz
//	SPCR = (1<<SPE)|(1<<MSTR)|(1<<SPR1)|(1<<SPR0);	 /* SPIA}X^[[hA64ɐݒ */
	SPCR = (1<<SPE)|(1<<MSTR);						 /* SPI enable, master mode, f=Fosc/2 */
//	SPCR = (1<<SPE)|(1<<MSTR)|(1<<SPR0);			 /* SPI enable, master mode, f=Fosc/16 */
	SPSR = (1<<SPI2X);								 /* SPICLK X1/2 */
//	SPSR = 0;										 /* SPICLK X1/2 */
								//SPI master mode, CPOL=0, CPHA=0, SCK Frequency = 8MHz/64=125kHz
								// SCK Frequency = 8MHz/2=4MHz
								//------------------------------------------------
								// b7:SPIE	0:SPI int16_t disable(0)/enable(1)
								// b6:SPE	1:SPI enable(1)/disable(0)
								// b5:DORD	0:MSB first(0)/LSB first(1)
								// b4:MSTR	1:SPI master(1)/slave(0) mode
								// b3:CPOL	0:Clock polarity rising edge(0)/falling edge(1)
								// b2:CPHA	1:Clock phase (1:leading edge: sampling, trailing edge : setup)
								// b1:SPR1	0
								// b0:SPR0	0
								//---------------------------------------------
								//		SPI2X	SPR1	SPR0	SCK Frequency
								//		0		0		0		fosc/4
								//		0		0		1		fosc/16
								//		0		1		0		fosc/64
								//		0		1		1		fosc/128
								//		1		0		0		fosc/2
								//		1		0		1		fosc/8
								//		1		1		0		fosc/32
								//		1		1		1		fosc/64
								//-------------------------------------------------


//	PORTD &= ~(1<<PD_LED1);	// PD_LED1o
//	PORTB &= ~(1<<PB_LED2);	// PB_LED2o

	ADMUX = (1<<REFS0)|7;	// ADC ref is  AVCC, A/D ch7

	TCCR2A = 1<<WGM21;						// Compare and match
	TCCR2B = (1<<CS22)|(1<<CS21)|(1<<CS20);	// 1/1024 prescaler 
//	OCR2A = 78-1;		// Set timer count value(8MHz/1024/78=100.16Hz)
//	OCR2A = 39-1;		// Set timer count value(4MHz/1024/9=108.5Hz)
	OCR2A = 20-1;		// Set timer count value(2MHz/1024/20=97.7Hz)
//	OCR2A = 98-1;		// Set timer count value(1MHz/1024/98=9.9Hz)
//	OCR2A = 9-1;		// Set timer count value(1MHz/1024/9=108.5Hz)
	TIMSK2 = 1<<OCIE2A;	// Compare and match interrupt enable

	SMCR = 1<<SE;			// VS1011_DREQ҂ԏȓd͂̂SleepԂƂ
//	PCICR = 1<<PCIE1;
//	PCMSK1 = (1<<PCINT8)|(1<<PCINT9)|(1<<PCINT11);	// Detect change of Key_Right, key_left, VS1011_DREQ
	PCMSK1 = (1<<PCINT11);	// VS1011_DREQ̃sωSleepԂ畜A


	sei();

	Timer = 0;
	while (Timer < 20) {			// 0.2bԃL[͂Ď
		if ( ! KeyRight() && !KeyLeft()  )
			goto fin;				// ĎԓɃL[͂ȂȂ΃mCYƂ݂Ȃăp[It
	}

	if (KeyRight() && KeyLeft())
		init_eeprom();				// EEPROM

	while ( KeyRight() || KeyLeft() ) 	// L[܂ő҂
		;

	KeyStatus = 0;


	restore_eeprom(&Song);			// EEPROMep[^ǂݏo

	if (Volume == 0xFF) {			// Volume̒l0xFFȂCPŨtbVvOꂽȂ̂
		init_eeprom();						// EEPROM
		write_eeprom(EEPROM_Volume, 0x30);	// ʂinit_eepromŃZbgȂ
		write_eeprom(EEPROM_Loud, 0);		// EhlXԍinit_eepromŃZbgȂ
		restore_eeprom(&Song);				// EEPROM珉ꂽp[^߂ēǂݍ
	}

	restore_loud();					// Loudnedd extentiontbVVS1011Ƀ[h

//	beep(1, BEEP_HIGH);
//	beep(1, BEEP_HIGH);

	if (SD_Init() == 0xFF){			// SDJ[hBJ[hZbgĂȂ0xFFԂ
		beep(3, BEEP_LOW);			// Rr[v炵
		goto fin;					// ̂ƃX[v
	}

	if (read_FAT16_info(&Song)){	// FAT16 p[^ǂݏoG[Ԃꂽꍇ
		beep(2, BEEP_LOW);			// Qr[v炵
		goto fin;					// ̂ƃX[v
	}

	Song.TotalSong = search_MP3(0xFFFF);	// ׂĂMP3t@CT
	if (Song.TotalSong == 0){		// MP3t@CȂꍇ
		beep(1, BEEP_LOW);			// Pr[v
		goto fin;					// ̂ƃX[v
	}

	search_MP3(Song.SongNum);		// J[hꂽǂSongNumԖڂ̋Ȃ̃oCgrĒׂ
	if (Song.SongSize !=				// SDJ[hlEEPROM̒lrĈvȂꍇ
		((uint32_t)Buff[28] + ((uint32_t)Buff[29] << 8)
		 + ((uint32_t)Buff[30] << 16) + ((uint32_t)Buff[31] << 24))){

		init_eeprom();					// J[hꂽ̂EEPROM
		restore_eeprom(&Song);			// EEPROM珉ꂽp[^ǂݍ
	}

	srand(Timer);					// _֐̎𖈉ς

	if (PlayMode & (1<<MOD_RND)) {
		Song.SongNum = rand() / (RAND_MAX / Song.TotalSong+1) + 1;
		Song.Cluster = 0;			// Ȃ̍ŏĐ
	}


//----------- ȍĐ̃[vJn -----------------

	for(;;){

		r = play_song(&Song);

		if (r < 0){						// ȍĐ̃G[
			beep(-r, BEEP_LOW);
			break;							//[vIăX[v

		} else if (r == 5){				// {[_EłShutdown
			beep(1, BEEP_HIGH);
			while ( KeyRight() || KeyLeft()  )		// WOvbṼ[X҂
				;
			break;							// [vIăX[v

		} else if (r == 1){				// WOvbVłShutdown
			save_eeprom(&Song);				// e[hp[^L
			beep(1, BEEP_HIGH);
			while ( KeyRight() || KeyLeft()  )		// WOvbṼ[X҂
				;
			break;							// [vIăX[v

		} else if (r == 2){				// Ȃi߂
			if (++Song.SongNum > Song.TotalSong) {	// Ō̋ȂȂ
				Song.SongNum = Song.TotalSong;			// WO1vbVIȂ̏ꍇŌ̋ȕς炸
				PlayMode &= ~(1<<MOD_SEARCH_FF);		// Intro search̏Ă
				SearchLen = 0;
			}
		} else if (r == 3){				// Ȃ߂
			if (--Song.SongNum == 0) {				// ŏ̋ȂȂ
				Song.SongNum = 1;						// WO1vbVIȂ̏ꍇŏ̋ȕς炸
				PlayMode &= ~(1<<MOD_SEARCH_REW);		// Intro search̏Ă
				SearchLen = 0;
			}
		} else if (r == 0){				// ȋȏI
			if (PlayMode & (1<<MOD_RND)) {			// _[h̏ꍇAȂ𑱂Ȃ
				while ( Song.SongNum == (c = rand() / (RAND_MAX / Song.TotalSong+1) + 1));
				Song.SongNum = c;
			} else if (!(PlayMode & (1<<MOD_REP_ONE))) {	// s[głȂꍇ
				Song.SongNum++;								// ȂЂƂi߂
			}
		} else if (r == 4){				// Intro search̋Ȍ
			PlayMode &= ~(1<<MOD_SEARCH_FF);	// Intro searchɂȂ
			PlayMode &= ~(1<<MOD_SEARCH_REW);
			SearchLen = 0;
			KeyStatus &= ~(1<<SW_BOTH);
			KeyMode = 0;
		}

		while (KeyMode == 1){				// WO1vbV̑I
			if (!SwBothCount) {
				if (SwRightCount > 2) {			// WOEȂȂi߂
					beep(1, BEEP_HIGH);
					SwRightCount = 0;
					if (++Song.SongNum > Song.TotalSong)
						Song.SongNum = Song.TotalSong;
				} else if (SwLeftCount > 2) {	// WOȂȂ߂
					beep(1, BEEP_HIGH);
					SwLeftCount = 0;
					if (--Song.SongNum == 0)
						Song.SongNum = 1;
				}
			}
		}

		if (Song.SongNum > Song.TotalSong) {	// Ō̋ȂĐꍇ
			if (PlayMode & (1<<MOD_REP_ALL))	// s[gI[[h̏ꍇ
				Song.SongNum = 1;				// ŏ̋Ȃɖ߂
			else								// ȊO
				break;							// [voăX[v
		}
		Song.Cluster = 0;						// Ȃ̍ŏĐ
	}

//----------- ȍĐ̃[vI -----------------

fin:					// EIb`hbOIɋNĂbotZbg
	write_eeprom(EEPROM_Timer, Timer);		// _[ĥ߂̎Timer炤
	cli();						// ݋֎~
	wdt_reset();				// EIb`hbO^C}@Zbg
	wdt_enable(WDTO_15MS);		// 15msecĎԊuZbgJn
	sei();						// ݋
	for (;;)
		;
}


// --------------------------- Interrupt routines -----------------------------


//*********************************
//	TimerCount 2 compare match A handler
//	100Hz
//*********************************
SIGNAL(SIG_OUTPUT_COMPARE2A) {

	Timer++;

	if (KeyMode != 0){	// KeyMode0ȊO̎XCb`͂1.5bȂKeyMode0ɃZbg
		if (!SwLeftCount && !SwRightCount) {
			if (--ModeTimeout == 0)
				KeyMode = 0;
		} else
			ModeTimeout = 150;					// Mode keep time is 1.5 sec
	}

	if (!(PINC & (1<<PC_SW_LEFT))) {
		if (SwLeftCount++ == 255)
				SwLeftCount = 255;
	 } else
		SwLeftCount = 0;

	if (!(PINC & (1<<PC_SW_RIGHT))) {
		if (SwRightCount++ == 255)
			SwRightCount = 255;
	} else
		SwRightCount = 0;

	if (SwLeftCount && SwRightCount) {
		if (SwBothCount++ == 255) {
			SwBothCount = 255;
		} else if (SwBothCount == 4) {
			KeyStatus |= (1<<SW_BOTH);
			SwReleaseCount = 4;
		} else if (SwBothCount > 50)				// WOXCb`Pbԉꂽ?
			KeyStatus |= (1<<SW_START_STOP);		// Yes
	} else
		SwBothCount = 0;


	if (KeyStatus & (1<<SW_START_STOP))
		return;

	if (KeyStatus & (1<<SW_BOTH)) {
		if (!SwLeftCount && !SwRightCount) {		// Was switch released for 8 times?
			if (--SwReleaseCount == 0) {			// Yes
				KeyStatus &= ~(1<<SW_BOTH);
				ModeTimeout = 150;					// Mode keep time is 1.5 sec
				if (++KeyMode > 8)
					KeyMode = 0;
				else
					beep(KeyMode, BEEP_HIGH);
			}
		}
	}


//*******************************************************
// mode 0: key_right -> Vol up			key_left -> Vol down
// mode 1: key_right -> Foward step		key_left -> Backword step
// mode 2: key_right -> FF				key_left -> REW
// mode 3: key_right -> Intro search on	key_left -> Intro search off
// mode 4: key_right -> Random mode		key_left -> Normal mode
// mode 5: key_right -> Repeat song		key_left -> Single play
// mode 6: key_right -> Repeat all		key_left -> Single play
// mode 7: key_right -> Speed up		key_left -> Speed down
// mode 8: key_right -> Loudness on		key_left -> Loudness off
// mode 9: key_right -> Battery check
//*******************************************************
	if (KeyMode == 0) {
		if (SwLeftCount > 1) {
			SwLeftCount = 0;
			if (Volume++ == 0xFF)
				Volume = 0xFF;
		} else if (SwRightCount > 1) {
			SwRightCount = 0;
			if (Volume-- == 0)
				Volume = 0;
		}
	}

	//  Start A/D converter prescaler 2MHz/64 = 31.3kHz
	ADCSRA = (1<<ADEN)|(1<<ADSC)|(1<<ADIE)|6;
}


//*********************************
//	A/D converter interrupt handler
//*********************************
SIGNAL(SIG_ADC) {
	int16_t		v;

	v = (int16_t)ADCL;
	Vbatt = (((int16_t)ADCH & 3) << 8);
	Vbatt |= v;

	cbi(ADCSRA, ADEN);	// A/D POWER OFF
}


//************************************************
// pin change interrupt
//
//		SW_RIGHT (PC0, PCINT8) pin change interrupt
//		SW_LEFT (PC1, PCINT9) pin change interrupt
//		VS1011_DREQ (PC3, PCINT11)
//************************************************
SIGNAL(SIG_PIN_CHANGE1) {
}


// --------------------------------- end of FIJI.C ---------------------------------
